-- Create a table to store the login sessions of users.

CREATE TABLE login_sessions (
    login_session_id SERIAL NOT NULL PRIMARY KEY,
    login_session_token_hash BYTEA NOT NULL UNIQUE,
    user_id UUID NOT NULL REFERENCES users(user_id),
    started_at_utc TIMESTAMP NOT NULL,
    xsrf_secret BYTEA NOT NULL
);

CREATE INDEX ON login_sessions (user_id);

COMMENT ON TABLE login_sessions IS 'A login session is equivalent to one browser session where the user has logged in. The session is identified by a cookie.';

COMMENT ON COLUMN login_sessions.login_session_id IS 'Synthetic numeric ID for the session, used for cross-referencing only. Should not be public.';

COMMENT ON COLUMN login_sessions.login_session_token_hash IS 'BLAKE2s-256 hash of the login session token (session cookie value).';

COMMENT ON COLUMN login_sessions.user_id IS 'ID of the user that this login session is for.';

COMMENT ON COLUMN login_sessions.started_at_utc IS 'Timestamp in UTC when the session was started.';

COMMENT ON COLUMN login_sessions.xsrf_secret IS 'Key for a Blake2sMac256 which is used to prevent Cross-Site Request Forgery.';
