use rstest::rstest;

use crate::cli::{handle_user_command, UserCommand};

use super::basic_system;

#[rstest]
#[tokio::test]
async fn test_cli_add_user() {
    let sys = basic_system().await;

    handle_user_command(
        UserCommand::Add {
            username: "jonathan".to_owned(),
            locked: true,
        },
        &sys.config,
        &sys.store,
    )
    .await
    .unwrap();

    let _: () = sys
        .store
        .txn(|mut txn| {
            Box::pin(async move {
                let user = txn.lookup_user_by_name("jonathan".to_owned()).await?;

                assert!(user.unwrap().locked);
                Ok(())
            })
        })
        .await
        .unwrap();
}

#[rstest]
#[tokio::test]
async fn test_cli_lock_and_unlock_user() {
    let sys = basic_system().await;

    handle_user_command(
        UserCommand::Add {
            username: "jonathan".to_owned(),
            locked: false,
        },
        &sys.config,
        &sys.store,
    )
    .await
    .unwrap();

    let _: () = sys
        .store
        .txn(|mut txn| {
            Box::pin(async move {
                let user = txn.lookup_user_by_name("jonathan".to_owned()).await?;

                assert!(!user.unwrap().locked);
                Ok(())
            })
        })
        .await
        .unwrap();

    handle_user_command(
        UserCommand::Lock {
            username: "jonathan".to_owned(),
        },
        &sys.config,
        &sys.store,
    )
    .await
    .unwrap();

    let _: () = sys
        .store
        .txn(|mut txn| {
            Box::pin(async move {
                let user = txn.lookup_user_by_name("jonathan".to_owned()).await?;

                assert!(user.unwrap().locked);
                Ok(())
            })
        })
        .await
        .unwrap();

    handle_user_command(
        UserCommand::Unlock {
            username: "jonathan".to_owned(),
        },
        &sys.config,
        &sys.store,
    )
    .await
    .unwrap();

    let _: () = sys
        .store
        .txn(|mut txn| {
            Box::pin(async move {
                let user = txn.lookup_user_by_name("jonathan".to_owned()).await?;

                assert!(!user.unwrap().locked);
                Ok(())
            })
        })
        .await
        .unwrap();
}

#[rstest]
#[tokio::test]
async fn test_cli_del_user() {
    let sys = basic_system().await;

    handle_user_command(
        UserCommand::Add {
            username: "jonathan".to_owned(),
            locked: true,
        },
        &sys.config,
        &sys.store,
    )
    .await
    .unwrap();

    let _: () = sys
        .store
        .txn(|mut txn| {
            Box::pin(async move {
                let user = txn.lookup_user_by_name("jonathan".to_owned()).await?;

                assert!(user.unwrap().locked);
                Ok(())
            })
        })
        .await
        .unwrap();

    handle_user_command(
        UserCommand::Delete {
            username: "jonathan".to_owned(),
        },
        &sys.config,
        &sys.store,
    )
    .await
    .unwrap();

    let _: () = sys
        .store
        .txn(|mut txn| {
            Box::pin(async move {
                let user = txn.lookup_user_by_name("jonathan".to_owned()).await?;

                assert!(user.is_none());
                Ok(())
            })
        })
        .await
        .unwrap();
}
